@@ -23,6 +23,9 @@ from scipy.interpolate import RectBivariateSpline
 
 import xraylib
 
+import matplotlib.pyplot as plt
+import matplotlib.cm as cm
+
 class HybridNotNecessaryWarning(Exception):
     def __init__(self, *args, **kwargs):
         pass
@@ -406,7 +409,7 @@ def hy_check_congruence(input_parameters=HybridInputParameters(), calculation_pa
 
             # REQUEST FILTERING OR REFUSING
 
-            if  not input_parameters.ghy_calcType in [3, 4, 6]: # no figure/thickness errors
+            if not input_parameters.ghy_calcType in [3, 4, 6]: # no figure/thickness errors
                 if input_parameters.ghy_automatic == 1:
                     if input_parameters.ghy_diff_plane == 1 and calculation_parameters.beam_not_cut_in_x :
                         calculation_parameters.ff_beam = input_parameters.original_shadow_beam
@@ -763,7 +766,6 @@ def hy_init(input_parameters=HybridInputParameters(), calculation_parameters=Hyb
         calculation_parameters.zz_focal_ray = copy.deepcopy(calculation_parameters.zz_screen) + \
                                               input_parameters.ghy_focallength * numpy.tan(calculation_parameters.dz_ray)
 
-
     t_image = calculation_parameters.shadow_oe_end._oe.T_IMAGE
 
     if input_parameters.ghy_distance < 0:
@@ -1547,10 +1549,12 @@ def propagate_2D(calculation_parameters, input_parameters):
                                                                 y_max=calculation_parameters.ghy_z_max)
 
         try:
-            for i in range(0, len(wavefront.electric_field_array.x_coord)):
-                for j in range(0, len(wavefront.electric_field_array.y_coord)):
-                    interpolated = calculation_parameters.wIray_2d.interpolate_value(wavefront.electric_field_array.x_coord[i],
-                                                                                     wavefront.electric_field_array.y_coord[j])
+            x_coord = wavefront.electric_field_array.x_coord
+            z_coord = wavefront.electric_field_array.y_coord
+
+            for i in range(0, len(x_coord)):
+                for j in range(0, len(z_coord)):
+                    interpolated = calculation_parameters.wIray_2d.interpolate_value(x_coord[i], z_coord[j])
                     wavefront.electric_field_array.set_z_value(i, j, numpy.sqrt(0.0 if interpolated < 0 else interpolated))
         except IndexError:
             raise Exception("Unexpected Error during interpolation: try reduce Number of bins for I(Tangential) histogram")
