@@ -44,14 +44,12 @@
 # ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE         #
 # POSSIBILITY OF SUCH DAMAGE.                                             #
 # ----------------------------------------------------------------------- #
-import time
-
-import numpy
+from typing import Tuple, Union
 
 from oasys.widgets import congruence
 from oasys.util.oasys_util import read_surface_file
 
-from orangecontrib.shadow.util.shadow_objects import ShadowBeam, ShadowOpticalElement
+from orangecontrib.shadow.util.shadow_objects import ShadowBeam, ShadowOpticalElement, ShadowCompoundOpticalElement
 from orangecontrib.shadow.util.shadow_util import ShadowPhysics, ShadowPreProcessor
 
 from hybrid_methods.coherence.hybrid_screen import *
@@ -63,14 +61,19 @@ IMPLEMENTATION = "SHADOW3"
 # -------------------------------------------------------------
 
 class ShadowHybridBeam(HybridBeamWrapper):
-    def __init__(self, beam : ShadowBeam, length_units):
-        super(ShadowHybridBeam, self).__init__(beam, length_units)
+    def __init__(self, beam : Union[ShadowBeam, Tuple[ShadowBeam, ShadowBeam]], length_units=HybridLengthUnits.MILLIMETERS, **kwargs):
+        super(ShadowHybridBeam, self).__init__(beam, length_units, **kwargs)
 
-    def duplicate(self): return ShadowHybridBeam(self.wrapped_beam.duplicate(), self.length_units)
+    def duplicate(self, **kwargs):
+        if isinstance(self.wrapped_beam, ShadowBeam): return ShadowHybridBeam(self.wrapped_beam.duplicate(**kwargs), self.length_units)
+        else:                                         return ShadowHybridBeam([wb.duplicate(**kwargs) for wb in self.wrapped_beam], self.length_units)
 
 class ShadowHybridOE(HybridOEWrapper):
-    def __init__(self, optical_element, name):
-        super(ShadowHybridOE, self).__init__(optical_element, name)
+    def __init__(self, optical_element: Union[Union[ShadowOpticalElement, ShadowCompoundOpticalElement], 
+                                              Tuple[Union[ShadowOpticalElement, ShadowCompoundOpticalElement], 
+                                                    Union[ShadowOpticalElement, ShadowCompoundOpticalElement]]], name, **kwargs):
+        if isinstance(optical_element, list): super(ShadowHybridOE, self).__init__(optical_element, name="KB" if name is None else name, **kwargs)
+        else:                                 super(ShadowHybridOE, self).__init__(optical_element, name=name, **kwargs)
 
     def check_congruence(self, calculation_type : int):
         if   calculation_type == HybridCalculationType.SIMPLE_APERTURE                and not ("ScreenSlits" in self.name):                       raise Exception("Simple Aperture calculation runs for Screen-Slits widgets only")
@@ -79,16 +82,27 @@ class ShadowHybridOE(HybridOEWrapper):
         elif calculation_type == HybridCalculationType.GRATING_SIZE_AND_ERROR_PROFILE and not ("Grating" in self.name):                           raise Exception("Grating calculation runs for Grating widgets only")
         elif calculation_type in [HybridCalculationType.CRL_SIZE, HybridCalculationType.CRL_SIZE_AND_ERROR_PROFILE] and \
                 not ("Lens" in self.name or "CRL" in self.name or "Transfocator" in self.name):                                                 raise Exception("CRL calculation runs for Lens, CRLs or Transfocators widgets only")
+        elif calculation_type in [HybridCalculationType.KB_SIZE, HybridCalculationType.KB_SIZE_AND_ERROR_PROFILE] and  not ("KB" in self.name): raise Exception("KB calculation runs for a pair of mirrors widgets only")
         
         shadow_oe = self.wrapped_optical_element
         
         if calculation_type in [HybridCalculationType.GRATING_SIZE_AND_ERROR_PROFILE, HybridCalculationType.MIRROR_SIZE_AND_ERROR_PROFILE]:
             if shadow_oe._oe.F_RIPPLE == 1 and shadow_oe._oe.F_G_S == 2: shadow_oe._oe.F_RIPPLE = 0 # disable slope error calculation for OE, must be done by HYBRID!
             else:                                                        raise Exception("O.E. has not Surface Error file (setup Advanced Option->Modified Surface:\n\nModification Type = Surface Error\nType of Defect: external spline)")
+        elif calculation_type == HybridCalculationType.KB_SIZE_AND_ERROR_PROFILE:
+            kb_1 = shadow_oe[0]
+            kb_2 = shadow_oe[1]
+            
+            if kb_1._oe.F_RIPPLE == 1 and kb_1._oe.F_G_S == 2: kb_1._oe.F_RIPPLE = 0 # disable slope error calculation for OE, must be done by HYBRID!
+            else:                                              raise Exception("KB1 has not Surface Error file (setup Advanced Option->Modified Surface:\n\nModification Type = Surface Error\nType of Defect: external spline)")
+            if kb_2._oe.F_RIPPLE == 1 and kb_2._oe.F_G_S == 2: kb_2._oe.F_RIPPLE = 0 # disable slope error calculation for OE, must be done by HYBRID!
+            else:                                              raise Exception("KB2 has not Surface Error file (setup Advanced Option->Modified Surface:\n\nModification Type = Surface Error\nType of Defect: external spline)")
         elif calculation_type == HybridCalculationType.MIRROR_OR_GRATING_SIZE: # ADDED BY XIANBO SHI
             shadow_oe._oe.F_RIPPLE = 0 # better safe than sorry
 
-    def duplicate(self): return ShadowHybridOE(self.wrapped_optical_element.duplicate(), self.name)
+    def duplicate(self, **kwargs):
+        if isinstance(self.wrapped_optical_element, list): return ShadowHybridOE([wo.duplicate(**kwargs) for wo in self.wrapped_optical_element], self.name)
+        else:                                              return ShadowHybridOE(self.wrapped_optical_element.duplicate(**kwargs), self.name)
 
 # -------------------------------------------------------------
 # HYBRID SCREENS HELPER CLASSES
@@ -456,7 +470,7 @@ class _ShadowOEWithSurfaceHybridScreen(_ShadowOEHybridScreen):
             oe_before._oe.FHIT_C = 0 # make it infinite to compute the size of the beam
             mirror_beam = ShadowBeam.traceFromOE(beam_before, oe_before, history=False)
             mirror_beam.loadFromFile("mirr." + str_n_oe)
-            oe_before._oe.FHIT_C = 1 # make it infinite to compute the size of the beam
+            oe_before._oe.FHIT_C = 1 # restore value
 
             geometrical_parameters.max_tangential = oe_before._oe.RLEN1 * to_m
             geometrical_parameters.min_tangential = oe_before._oe.RLEN2 * to_m
@@ -709,34 +723,71 @@ class _ShadowOELensAndErrorHybridScreen(_ShadowOELensHybridScreen):
 
         return ScaledMatrix(x_coords*coords_to_m, y_coords*coords_to_m, z_values.T*thickness_to_m)
 
+class _ShadowOEKBMirrorHybridScreen():
+
+    def _get_hybrid_beam_instance(self, input_parameters: HybridInputParameters, kb_mirror_input_beam):
+        return ShadowHybridBeam(beam=kb_mirror_input_beam, length_units=input_parameters.beam.length_units, history=True)
+
+    def _get_hybrid_oe_instance(self, input_parameters: HybridInputParameters, kb_mirror):
+        return ShadowHybridOE(optical_element=kb_mirror, name=input_parameters.beam.wrapped_beam[1].getOEHistory(-1)._widget_class_name)
+
+    def _modify_image_plane_distance_on_kb_1(self, kb_mirror_1: ShadowOpticalElement, kb_mirror_2: ShadowOpticalElement):
+        total_image_distance = kb_mirror_1._oe.T_IMAGE + \
+                               kb_mirror_2._oe.T_SOURCE + \
+                               kb_mirror_2._oe.T_IMAGE
+
+        kb_mirror_1._oe.T_IMAGE = total_image_distance
+
+    def _merge_beams(self, beam_1: ShadowHybridBeam, beam_2: ShadowHybridBeam):
+        if beam_1 is None or beam_2 is None: return None
+
+        go_beam_1 = numpy.where(beam_1.wrapped_beam._beam.rays[:, 9] == 1)
+        go_beam_2 = numpy.where(beam_2.wrapped_beam._beam.rays[:, 9] == 1)
+
+        if   len(go_beam_2[0]) < len(go_beam_1[0]): go_beam_1 = go_beam_1[0][0 : len(go_beam_2[0])]
+        elif len(go_beam_2[0]) > len(go_beam_1[0]): go_beam_2 = go_beam_2[0][0 : len(go_beam_1[0])]
+
+        beam_2.wrapped_beam._beam.rays[go_beam_2, 0] = beam_1.wrapped_beam._beam.rays[go_beam_1, 2] # tangential component 1 becomes the sagittal 2
+        beam_2.wrapped_beam._beam.rays[go_beam_2, 3] = beam_1.wrapped_beam._beam.rays[go_beam_1, 5]
+
+        return beam_2
 
 # -------------------------------------------------------------
 # HYBRID SCREENS IMPLEMENTATION CLASSES
 # -------------------------------------------------------------
 
 class ShadowSimpleApertureHybridScreen(_ShadowApertureHybridScreen, AbstractSimpleApertureHybridScreen):
-    def __init__(self, wave_optics_provider : HybridWaveOpticsProvider):
-        AbstractSimpleApertureHybridScreen.__init__(self, wave_optics_provider)
+    def __init__(self, wave_optics_provider : HybridWaveOpticsProvider, **kwargs):
+        AbstractSimpleApertureHybridScreen.__init__(self, wave_optics_provider, **kwargs)
 
 class ShadowMirrorOrGratingSizeHybridScreen(_ShadowOEWithSurfaceHybridScreen, AbstractMirrorOrGratingSizeHybridScreen):
-    def __init__(self, wave_optics_provider: HybridWaveOpticsProvider):
-        AbstractMirrorOrGratingSizeHybridScreen.__init__(self, wave_optics_provider)
+    def __init__(self, wave_optics_provider: HybridWaveOpticsProvider, **kwargs):
+        AbstractMirrorOrGratingSizeHybridScreen.__init__(self, wave_optics_provider, **kwargs)
 
 class ShadowMirrorSizeAndErrorHybridScreen(_ShadowOEWithSurfaceAndErrorHybridScreen, AbstractMirrorSizeAndErrorHybridScreen):
-    def __init__(self, wave_optics_provider: HybridWaveOpticsProvider):
-        AbstractMirrorSizeAndErrorHybridScreen.__init__(self, wave_optics_provider)
+    def __init__(self, wave_optics_provider: HybridWaveOpticsProvider, **kwargs):
+        AbstractMirrorSizeAndErrorHybridScreen.__init__(self, wave_optics_provider, **kwargs)
 
 class ShadowGratingSizeAndErrorHybridScreen(_ShadowOEWithSurfaceAndErrorHybridScreen, AbstractGratingSizeAndErrorHybridScreen):
-    def __init__(self, wave_optics_provider: HybridWaveOpticsProvider):
-        AbstractGratingSizeAndErrorHybridScreen.__init__(self, wave_optics_provider)
+    def __init__(self, wave_optics_provider: HybridWaveOpticsProvider, **kwargs):
+        AbstractGratingSizeAndErrorHybridScreen.__init__(self, wave_optics_provider, **kwargs)
 
 class ShadowCRLSizeHybridScreen(_ShadowOELensHybridScreen, AbstractCRLSizeHybridScreen):
-    def __init__(self, wave_optics_provider: HybridWaveOpticsProvider):
-        AbstractCRLSizeHybridScreen.__init__(self, wave_optics_provider)
+    def __init__(self, wave_optics_provider: HybridWaveOpticsProvider, **kwargs):
+        AbstractCRLSizeHybridScreen.__init__(self, wave_optics_provider, **kwargs)
 
 class ShadowCRLSizeAndErrorHybridScreen(_ShadowOELensAndErrorHybridScreen, AbstractCRLSizeAndErrorHybridScreen):
-    def __init__(self, wave_optics_provider: HybridWaveOpticsProvider):
-        AbstractCRLSizeAndErrorHybridScreen.__init__(self, wave_optics_provider)
+    def __init__(self, wave_optics_provider: HybridWaveOpticsProvider, **kwargs):
+        AbstractCRLSizeAndErrorHybridScreen.__init__(self, wave_optics_provider, **kwargs)
+
+class ShadowKBMirrorSizeHybridScreen(_ShadowOEKBMirrorHybridScreen, AbstractKBMirrorSizeHybridScreen):
+    def __init__(self, wave_optics_provider: HybridWaveOpticsProvider, implementation: str, **kwargs):
+        AbstractKBMirrorSizeHybridScreen.__init__(self, wave_optics_provider, implementation, **kwargs)
+
+class ShadowKBMirrorSizeAndErrorHybridScreen(_ShadowOEKBMirrorHybridScreen, AbstractKBMirrorSizeAndErrorHybridScreen):
+    def __init__(self, wave_optics_provider: HybridWaveOpticsProvider, implementation: str, **kwargs):
+        AbstractKBMirrorSizeAndErrorHybridScreen.__init__(self, wave_optics_provider, implementation, **kwargs)
+
 
 # -------------------------------------------------------------
 # -------------------------------------------------------------
@@ -754,164 +805,7 @@ try:
     hsm.add_hybrid_screen_class(IMPLEMENTATION, ShadowGratingSizeAndErrorHybridScreen)
     hsm.add_hybrid_screen_class(IMPLEMENTATION, ShadowCRLSizeHybridScreen)
     hsm.add_hybrid_screen_class(IMPLEMENTATION, ShadowCRLSizeAndErrorHybridScreen)
+    hsm.add_hybrid_screen_class(IMPLEMENTATION, ShadowKBMirrorSizeHybridScreen)
+    hsm.add_hybrid_screen_class(IMPLEMENTATION, ShadowKBMirrorSizeAndErrorHybridScreen)
 except Exception as e:
     print(e)
-
-'''
-OLD CODE TO BE KEPT FOR COMPARISONS
-'''
-'''
-    def _manage_common_initial_screen_projection_data_old(self, input_parameters: HybridInputParameters, calculation_parameters: AbstractHybridScreen.CalculationParameters):
-        self._check_compound_oe(input_parameters, calculation_parameters)
-
-        input_shadow_beam = calculation_parameters.get("shadow_beam")
-
-        # Before ray-tracing save the original history:
-        calculation_parameters.set("original_beam_history", input_shadow_beam.getOEHistory())
-
-        history_entry = input_shadow_beam.getOEHistory(input_shadow_beam._oe_number)
-    
-        shadow_oe            = history_entry._shadow_oe_start.duplicate()  # no changes to the original object!
-        shadow_oe_input_beam = history_entry._input_beam.duplicate(history=False)
-
-        hybrid_screen_file_name = self._set_hybrid_screen(input_parameters, shadow_oe)
-
-        # tracing must be done without o.e. movements: hybrid is going to take care of that
-        x_rot = shadow_oe._oe.X_ROT
-        y_rot = shadow_oe._oe.Y_ROT
-        z_rot = shadow_oe._oe.Z_ROT
-
-        shadow_oe._oe.X_ROT = 0.0
-        shadow_oe._oe.Y_ROT = 0.0
-        shadow_oe._oe.Z_ROT = 0.0
-
-        input_parameters.listener.status_message("Creating HYBRID screen: redo simulation with modified O.E.")
-
-        shadow_beam_at_image_plane = ShadowBeam.traceFromOE(shadow_oe_input_beam, shadow_oe, history=False)
-
-        # restore o.e. setting for further calculations
-        shadow_oe._oe.X_ROT = x_rot
-        shadow_oe._oe.Y_ROT = y_rot
-        shadow_oe._oe.Z_ROT = z_rot
-
-        calculation_parameters.set("shadow_beam", shadow_beam_at_image_plane)
-
-        image_beam, image_beam_lo = self._process_shadow_beam(shadow_beam_at_image_plane, lost=True)  # xshi change from 0 to 1
-        image_beam.set_initial_flux(input_parameters.original_beam.wrapped_beam.get_initial_flux())
-
-        calculation_parameters.set("shadow_oe_end",  shadow_oe)
-        calculation_parameters.set("image_plane_beam",  image_beam)
-        calculation_parameters.set("image_plane_beam_lost", image_beam_lo)
-
-        # read shadow screen file
-        screen_beam = self._read_shadow_file(hybrid_screen_file_name)
-
-        calculation_parameters.set("screen_plane_beam",  screen_beam)
-
-        energy     = ShadowPhysics.getEnergyFromShadowK(screen_beam._beam.rays[:, 10])     # eV
-        wavelength = ShadowPhysics.getWavelengthFromShadowK(screen_beam._beam.rays[:, 10]) # Å
-
-        input_parameters.listener.status_message("Using MEAN photon energy [eV]:" + str(numpy.average(energy)))
-
-        xx_screen = screen_beam._beam.rays[:, 0]
-        zz_screen = screen_beam._beam.rays[:, 2]
-        xp_screen = screen_beam._beam.rays[:, 3]
-        yp_screen = screen_beam._beam.rays[:, 4]
-        zp_screen = screen_beam._beam.rays[:, 5]
-
-        x_min     = numpy.min(xx_screen)
-        x_max     = numpy.max(xx_screen)
-        z_min     = numpy.min(zz_screen)
-        z_max     = numpy.max(zz_screen)
-        dx_rays   = numpy.arctan(xp_screen / yp_screen)  # calculate divergence from direction cosines from SHADOW file  dx = atan(v_x/v_y)
-        dz_rays   = numpy.arctan(zp_screen / yp_screen)  # calculate divergence from direction cosines from SHADOW file  dz = atan(v_z/v_y)
-
-        input_parameters.listener.error_message("{} {} {} {}".format(x_min, x_max, z_min, z_max))
-
-        to_m  = input_parameters.beam.length_units_to_m
-
-        calculation_parameters.energy     = numpy.average(energy)
-        calculation_parameters.wavelength = numpy.average(wavelength)*1e-10 # in m
-        calculation_parameters.xx_screen  = xx_screen * to_m
-        calculation_parameters.zz_screen  = zz_screen * to_m
-        calculation_parameters.xp_screen  = xp_screen * to_m
-        calculation_parameters.yp_screen  = yp_screen * to_m
-        calculation_parameters.zp_screen  = zp_screen * to_m
-        calculation_parameters.x_min      = x_min * to_m
-        calculation_parameters.x_max      = x_max * to_m
-        calculation_parameters.z_min      = z_min * to_m
-        calculation_parameters.z_max      = z_max * to_m
-        calculation_parameters.dx_rays    = dx_rays # radians
-        calculation_parameters.dz_rays    = dz_rays # radians
-
-        return calculation_parameters
-
-    def _set_hybrid_screen(self, input_parameters, shadow_oe):
-        # Set Hybrid Screen
-        if shadow_oe._oe.F_SCREEN == 1:
-            if shadow_oe._oe.N_SCREEN == 10: raise Exception("Hybrid Screen has not been created: O.E. has already 10 screens")
-
-            n_screen = shadow_oe._oe.N_SCREEN + 1
-            i_screen = shadow_oe._oe.I_SCREEN
-            sl_dis = shadow_oe._oe.I_ABS
-            i_abs = shadow_oe._oe.SL_DIS
-            i_slit = shadow_oe._oe.I_SLIT
-            i_stop = shadow_oe._oe.I_STOP
-            k_slit = shadow_oe._oe.K_SLIT
-            thick = shadow_oe._oe.THICK
-            file_abs = numpy.copy(shadow_oe._oe.FILE_ABS)
-            rx_slit = shadow_oe._oe.RX_SLIT
-            rz_slit = shadow_oe._oe.RZ_SLIT
-            cx_slit = shadow_oe._oe.CX_SLIT
-            cz_slit = shadow_oe._oe.CZ_SLIT
-            file_scr_ext = numpy.copy(shadow_oe._oe.FILE_SCR_EXT)
-
-            screen_index = n_screen - 1
-
-            i_screen[screen_index] = 0
-            sl_dis[screen_index] = 0
-            i_abs[screen_index] = 0
-            i_slit[screen_index] = 0
-            i_stop[screen_index] = 0
-            k_slit[screen_index] = 0
-            thick[screen_index] = 0
-            rx_slit[screen_index] = 0
-            rz_slit[screen_index] = 0
-            cx_slit[screen_index] = 0
-            cz_slit[screen_index] = 0
-        else:
-            n_screen = 1
-            i_screen = numpy.zeros(10)  # after
-            i_abs = numpy.zeros(10)
-            i_slit = numpy.zeros(10)
-            i_stop = numpy.zeros(10)
-            k_slit = numpy.zeros(10)
-            thick = numpy.zeros(10)
-            file_abs = numpy.array(['', '', '', '', '', '', '', '', '', ''])
-            rx_slit = numpy.zeros(10)
-            rz_slit = numpy.zeros(10)
-            sl_dis = numpy.zeros(10)
-            file_scr_ext = numpy.array(['', '', '', '', '', '', '', '', '', ''])
-            cx_slit = numpy.zeros(10)
-            cz_slit = numpy.zeros(10)
-
-            screen_index = 0
-        shadow_oe._oe.set_screens(n_screen,
-                                  i_screen,
-                                  i_abs,
-                                  sl_dis,
-                                  i_slit,
-                                  i_stop,
-                                  k_slit,
-                                  thick,
-                                  file_abs,
-                                  rx_slit,
-                                  rz_slit,
-                                  cx_slit,
-                                  cz_slit,
-                                  file_scr_ext)
-        self._fix_specific_oe_attributes(shadow_oe, input_parameters.optical_element.wrapped_optical_element, screen_index)
-
-        return "screen." + self._get_oe_string(input_parameters) + ("0" + str(n_screen)) if n_screen < 10 else "10"
-
-'''
\ No newline at end of file
